@@ -11,11 +11,8 @@
 #ifndef __XML_SCHEMA_INTERNALS_H__
 #define __XML_SCHEMA_INTERNALS_H__
 
-#if defined(WIN32) && defined(_MSC_VER)
-#include <libxml/xmlwin32version.h>
-#else
 #include <libxml/xmlversion.h>
-#endif
+
 #ifdef LIBXML_SCHEMAS_ENABLED
 
 #include <libxml/xmlregexp.h>
@@ -266,6 +263,8 @@ struct _xmlSchema {
     xmlHashTablePtr notaDecl;
 
     xmlHashTablePtr schemasImports;
+
+    void *_private;	/* unused by the library for users or bindings */
 };
 
 void	xmlSchemaFreeType	(xmlSchemaTypePtr type);
